package com.safeway.epe.offersdataproducer.producer;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.safeway.epe.offersdataproducer.appconfiguration.AppConfigs;
import com.safeway.epe.offersdataproducer.appconfiguration.ProducerConfiguration;
import com.safeway.epe.offersdataproducer.dtomodel.OffersDataDTO;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.Future;

@Component
public class OffersDataProducer {
    private static final Logger logger = LoggerFactory.getLogger(OffersDataProducer.class);
    @Autowired
    private ProducerConfiguration producerConfiguration;
    @Autowired
    private ObjectMapper objectMapper;

    public Future sendEvent(OffersDataDTO offersDataDTO) throws JsonProcessingException {
        logger.info("Start :: OffersDataProducer.sendEvent");
        String offersDataDTOResp = objectMapper.writeValueAsString(offersDataDTO);
        KafkaProducer producer =  new KafkaProducer(producerConfiguration.configuration());
        ProducerRecord record = new ProducerRecord(AppConfigs.TOPIC_NAME , offersDataDTOResp);
        Future<RecordMetadata> future = producer.send(record);
        logger.info("End :: OffersDataProducer.sendEvent");
        return future;
    }
}
