package com.safeway.epe.offersdataproducer.resource;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.safeway.epe.offersdataproducer.appconfiguration.AppConfigs;
import com.safeway.epe.offersdataproducer.domodel.*;
import com.safeway.epe.offersdataproducer.dtomodel.*;
import com.safeway.epe.offersdataproducer.producer.OffersDataProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.kafka.common.TopicPartition;
import org.json.JSONException;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;
import org.skyscreamer.jsonassert.Customization;
import org.skyscreamer.jsonassert.JSONAssert;
import org.skyscreamer.jsonassert.JSONCompareMode;
import org.skyscreamer.jsonassert.comparator.CustomComparator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.concurrent.SettableListenableFuture;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("test")
public class OffersDataProducerResourceComponentTest {
    private static final String FILE_PATH = "src/test/java/com/safeway/epe/offersdataproducer/resource/";
    @Autowired
    private OffersDataProducerResource offersDataProducerResource;
    @MockBean
    private OffersDataProducer offersDataProducer;
    @Autowired
    private  ObjectMapper objectMapper;
    @Test
    public void publishOfferEventWithArgumentCatureTest() throws JsonProcessingException, ExecutionException, InterruptedException {
        ObjectMapper mapper = new ObjectMapper();
        OffersDataDO offersDataDO = buildOffersDataDO();
        OffersDataDTO offersDataDTO = buildOffersDataDTO();
        String offersDataDTOResp = mapper.writeValueAsString(offersDataDTO);
        RecordMetadata recordMetadata = new RecordMetadata(new TopicPartition(AppConfigs.TOPIC_NAME, 0),
                8, 1, 342, 1111111L, -1, 581);
        SettableListenableFuture future = new SettableListenableFuture();
        future.set(recordMetadata);
        ArgumentCaptor<OffersDataDTO> serviceRequestArgumentCaptor = ArgumentCaptor.forClass(OffersDataDTO.class);
        ProducerRecord record = new ProducerRecord(AppConfigs.TOPIC_NAME ,0, 1 ,offersDataDTOResp);
        Mockito.when(offersDataProducer.sendEvent(serviceRequestArgumentCaptor.capture())).thenReturn(future);
        ResponseEntity<Future<RecordMetadata>> result = offersDataProducerResource.publishOfferEvent(offersDataDO);
        OffersDataDTO inputDataResp = serviceRequestArgumentCaptor.getValue();
        validateRequest( serviceRequestArgumentCaptor,"OffersDataDTO.json");
    }
    private String retrieveExpectedResult(String fileName) {
        StringBuilder sb = new StringBuilder();
        try (BufferedReader br = new BufferedReader(new FileReader(FILE_PATH + fileName))) {
            String line;
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return sb.toString();
    }
    private void validateRequest(ArgumentCaptor<OffersDataDTO> argumentCaptor, String fileName) throws JsonProcessingException {
        String actual = objectMapper.writeValueAsString(argumentCaptor.getValue());
        String expectedServiceRequest = retrieveExpectedResult(fileName);
        try {
            JSONAssert.assertEquals(actual,expectedServiceRequest,
                    new CustomComparator(JSONCompareMode.STRICT,
                            new Customization("offerId" ,(value1 , value2) -> value1.equals(value2)) ,
                            new Customization("offersDTO[0].infoDTO.idDTO.offerId" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].infoDTO.offerId" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.benefitDTO.benefitValueType" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.benefitDTO.discountDTO[0].allowNegative" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.benefitDTO.discountDTO[0].discountTierDTO[0].upTo" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.benefitDTO.discountDTO[0].flexNegative" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.endDate" ,(value1 , value2) -> true ),
                            new Customization("offersDTO[0].rulesDTO.offerId" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.startDate" ,(value1 , value2) -> true ),
                            new Customization("offersDTO[0].infoDTO.idDTO.offerId" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].infoDTO.offerId" ,(value1 , value2) -> value1.equals(value2) ),
                            new Customization("offersDTO[0].rulesDTO.benefitDTO.benefitValueType" ,(value1 , value2) -> value1.equals(value2) )
                    )
            );
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }
    private OffersDataDO buildOffersDataDO() {
        DiscountTierDO discountTierDO = DiscountTierDO
                .builder().amount(6.99f).upTo("3")
                .itemLimit(5).weightLimit(2).build();
        PointsDO pointsDO = PointsDO.builder().generalPoints(5).loyaltyPoints(250).build();
        DiscountDO discountDO = DiscountDO
                .builder().discountTierDO(Lists.newArrayList(discountTierDO))
                .allowNegative(false).flexNegative(false).build();
        BenefitDO benefitDO = BenefitDO
                .builder().discountDO(Lists.newArrayList(discountDO))
                .pointsDO(Lists.newArrayList(pointsDO)).benefitValueType("A")
                .build();
        IdDO idDO = IdDO.builder().manufacturerId("MMM").offerId(135918555).build();
        InfoDO infoDO = InfoDO
                .builder().offerId(135918555).idDO(idDO).offerProgramCode("SC")
                .terminalsDO(Lists.newArrayList("Groceryworks (Safeway.com)",
                        "Smart Cart (QVS Virtual Terminals)",
                        "Bakery")).build();
        RulesDO rulesDO = RulesDO.builder().offerId(135918555).benefitDO(benefitDO).endDate("2021-02-27T06:59:58.000+00:00")
                .startDate("2019-07-27T07:00:00.000+00:00").build();
        OffersDO offersDO = OffersDO.builder().infoDO(infoDO).rulesDO(rulesDO).build();
        OffersDataDO offersDataDO = OffersDataDO.builder().offerId(135918555).offersDO(Lists.newArrayList(offersDO)).build();

        return offersDataDO;
    }
    private OffersDataDTO buildOffersDataDTO() {
        DiscountTierDTO discountTierDTO = DiscountTierDTO
                .builder().amount(6.99f).upTo("3")
                .itemLimit(5).weightLimit(2).build();
        PointsDTO pointsDTO = PointsDTO.builder().generalPoints(5).loyaltyPoints(250).build();
        DiscountDTO discountDTO = DiscountDTO
                .builder().discountTierDTO(Lists.newArrayList(discountTierDTO))
                .allowNegative(false).flexNegative(false).build();
        BenefitDTO benefitDTO = BenefitDTO
                .builder().discountDTO(Lists.newArrayList(discountDTO))
                .pointsDTO(Lists.newArrayList(pointsDTO)).benefitValueType("A")
                .build();
        IdDTO idDTO = IdDTO.builder().manufacturerId("MMM").offerId(135918555).build();
        InfoDTO infoDTO = InfoDTO
                .builder().offerId(135918555).idDTO(idDTO).offerProgramCode("SC")
                .terminalsDTO(Lists.newArrayList("Groceryworks (Safeway.com)",
                        "Smart Cart (QVS Virtual Terminals)",
                        "Bakery")).build();
        RulesDTO rulesDTO = RulesDTO.builder().offerId(135918555).benefitDTO(benefitDTO).endDate("2021-02-27T06:59:58.000+00:00")
                .startDate("2019-07-27T07:00:00.000+00:00").build();
        OffersDTO offersDTO = OffersDTO.builder().infoDTO(infoDTO).rulesDTO(rulesDTO).build();
        OffersDataDTO offersDataDTO = OffersDataDTO.builder().offerId(135918555).offersDTO(Lists.newArrayList(offersDTO)).build();

        return offersDataDTO;
    }
}
